import enum
import weakref
import functools

import cassiopeia.type.api.exception


class CassiopeiaObject(object):
    """
    An object storing data from the API, with various helpful utilities and shortcuts
    """

    def __init__(self, data):
        """
        An object storing data from the API, with various helpful utilities and shortcuts
        """
        if data.__class__ is not self.dto_type:
            raise cassiopeia.type.api.exception.CassiopeiaException("Tried to instantiate a core {class_} with a {dto} dto!".format(class_=self.__class__.__name__, dto=data.__class__.__name__))
        self.data = data

    def to_json(self, **kwargs):
        """
        Args:
            data (CassiopeiaDto): the underlying DTO object with the data for this type
        """
        return self.data.to_json(**kwargs)

    def __str__(self):
        return str(self.data)

    def __eq__(self, other):
        return self.data == other.data

    def __ne__(self, other):
        return self.data != other.data

    def __hash__(self):
        return hash(id(self))


class LazyProperty(object):
    def __init__(self, method):
        """
        Gets a JSON representation of the object

        Returns:
            str: a JSON representation of the object
        """
        self.method = method
        self.values = weakref.WeakKeyDictionary()

    def __set__(self, obj, value):
        raise AttributeError("can't set attribute")

    def __delete__(self, obj):
        raise AttributeError("can't delete attribute")

    def __get__(self, obj, t=None):
        try:
            return self.values[obj]
        except KeyError:
            self.values[obj] = self.method(obj)
            return self.values[obj]


def lazyproperty(method):
    """
    Args:
        method (function): the method to turn into a lazy property
    """
    prop = LazyProperty(method)

    @property
    @functools.wraps(method)
    def lazy(self):
        return prop.__get__(self)

    return lazy


class immutablemethod(object):
    """
    Makes a property load only once and store the result value to be returned to all later calls

    Args:
        method (function): the method to turn into a lazy property

    Returns:
        function: the method as a lazy property
    """

    def __init__(self, method):
        """
        Makes a method un-deletable and un-repleacable
        @decorator
        """
        self.method = method

    def __set__(self, obj, value):
        raise AttributeError("can't set method")

    def __delete__(self, obj):
        raise AttributeError("can't delete method")

    def __get__(self, obj, type=None):
        @functools.wraps(self.method)
        def curried(*args, **kwargs):
            return self.method(obj, *args, **kwargs)
        return curried


def inheritdocs(class_):
    """
    Args:
        method (function): the method to make immutable
    """
    for name, method in vars(class_).items():
        if not method.__doc__:
            for parent in class_.__bases__:
                try:
                    p_method = getattr(parent, name)
                    if p_method and p_method.__doc__:
                        method.__doc__ = p_method.__doc__
                except AttributeError:
                    continue
    return class_


class LoadPolicy(enum.Enum):
    lazy = "LAZY"
    eager = "EAGER"


class MasteryType(enum.Enum):
    ferocity = "Ferocity"
    resolve = "Resolve"
    cunning = "Cunning"


class Lane(enum.Enum):
    top_lane = "TOP"
    jungle = "JUNGLE"
    mid_lane = "MIDDLE"
    bot_lane = "BOTTOM"

    def for_id(id_):
        try:
            return Lane.by_id[id_]
        except:
            return None


Lane.by_id = {
    1: Lane.top_lane,
    2: Lane.mid_lane,
    3: Lane.jungle,
    4: Lane.bot_lane
}


class Role(enum.Enum):
    none = "NONE"
    solo = "SOLO"
    duo = "DUO"
    carry = "DUO_CARRY"
    support = "DUO_SUPPORT"

    def for_id(id_):
        try:
            return Role.by_id[id_]
        except:
            return None


Role.by_id = {
    1: Role.duo,
    2: Role.support,
    3: Role.carry,
    4: Role.solo
}


class Side(enum.Enum):
    blue = 100
    red = 200


class Queue(enum.Enum):
    custom = "CUSTOM"
    normal_blind_threes = "NORMAL_3x3"
    normal_blind_fives = "NORMAL_5x5_BLIND"
    normal_draft_fives = "NORMAL_5x5_DRAFT"
    ranked_solo = "RANKED_SOLO_5x5"
    ranked_premade_fives = "RANKED_PREMADE_5x5"
    ranked_premade_threes = "RANKED_PREMADE_3x3"
    ranked_threes = "RANKED_TEAM_3x3"
    ranked_fives = "RANKED_TEAM_5x5"
    dominion_blind = "ODIN_5x5_BLIND"
    dominion_draft = "ODIN_5x5_DRAFT"
    bot_dominion = "BOT_ODIN_5x5"
    bot_fives = "BOT_5x5"
    bot_intro_fives = "BOT_5x5_INTRO"
    bot_beginner_fives = "BOT_5x5_BEGINNER"
    bot_intermediate_fives = "BOT_5x5_INTERMEDIATE"
    bot_threes = "BOT_TT_3x3"
    team_builder = "GROUP_FINDER_5x5"
    aram = "ARAM_5x5"
    one_for_all = "ONEFORALL_5x5"
    showdown_solo = "FIRSTBLOOD_1x1"
    showdown_duo = "FIRSTBLOOD_2x2"
    hexakill_summoners_rift = "SR_6x6"
    urf = "URF_5x5"
    one_for_all_mirror = "ONEFORALL_MIRRORMODE_5x5"
    bot_urf = "BOT_URF_5x5"
    doom_bots_1 = "NIGHTMARE_BOT_5x5_RANK1"
    doom_bots_2 = "NIGHTMARE_BOT_5x5_RANK2"
    doom_bots_5 = "NIGHTMARE_BOT_5x5_RANK5"
    ascension = "ASCENSION_5x5"
    hexakill_twisted_treeline = "HEXAKILL"
    butchers_bridge = "BILGEWATER_ARAM_5x5"
    poro_king = "KING_PORO_5x5"
    nemesis_draft = "COUNTER_PICK"
    black_market = "BILGEWATER_5x5"
    nexus_siege = "SIEGE"
    definitely_not_dominion = "DEFINITELY_NOT_DOMINION_5x5"
    random_urf = "ARURF_5X5"
    dynamic_queue = "TEAM_BUILDER_DRAFT_UNRANKED_5x5"
    ranked_dynamic_queue = "TEAM_BUILDER_DRAFT_RANKED_5x5"
    ranked_solo_queue = "TEAM_BUILDER_RANKED_SOLO"
    flex = "RANKED_FLEX_SR"
    flex_threes = "RANKED_FLEX_TT"


    def for_id(id_):
        try:
            return Queue.by_id[id_]
        except:
            return None


Queue.by_id = {
    0: Queue.custom,
    8: Queue.normal_blind_threes,
    2: Queue.normal_blind_fives,
    14: Queue.normal_draft_fives,
    4: Queue.ranked_solo,
    6: Queue.ranked_premade_fives,
    9: Queue.flex_threes,
    41: Queue.ranked_threes,
    42: Queue.ranked_fives,
    16: Queue.dominion_blind,
    17: Queue.dominion_draft,
    25: Queue.bot_dominion,
    7: Queue.bot_fives,
    31: Queue.bot_intro_fives,
    32: Queue.bot_beginner_fives,
    33: Queue.bot_intermediate_fives,
    52: Queue.bot_threes,
    61: Queue.team_builder,
    65: Queue.aram,
    70: Queue.one_for_all,
    72: Queue.showdown_solo,
    73: Queue.showdown_duo,
    75: Queue.hexakill_summoners_rift,
    76: Queue.urf,
    78: Queue.one_for_all_mirror,
    83: Queue.bot_urf,
    91: Queue.doom_bots_1,
    92: Queue.doom_bots_2,
    93: Queue.doom_bots_5,
    96: Queue.ascension,
    98: Queue.hexakill_twisted_treeline,
    100: Queue.butchers_bridge,
    300: Queue.poro_king,
    310: Queue.nemesis_draft,
    313: Queue.black_market,
    315: Queue.nexus_siege,
    317: Queue.definitely_not_dominion,
    318: Queue.random_urf,
    400: Queue.dynamic_queue,
    410: Queue.ranked_dynamic_queue,
    420: Queue.ranked_solo_queue,
    440: Queue.flex
}
ranked_queues = {Queue.ranked_solo, Queue.ranked_threes, Queue.ranked_fives, Queue.ranked_dynamic_queue, Queue.flex, Queue.flex_threes, Queue.ranked_solo_queue}


class Tier(enum.Enum):
    challenger = "CHALLENGER"
    master = "MASTER"
    diamond = "DIAMOND"
    platinum = "PLATINUM"
    gold = "GOLD"
    silver = "SILVER"
    bronze = "BRONZE"
    unranked = "UNRANKED"


class Division(enum.Enum):
    one = "I"
    two = "II"
    three = "III"
    four = "IV"
    five = "V"


class Season(enum.Enum):
    preseason_3 = "PRESEASON3"
    season_3 = "SEASON3"
    preseason_4 = "PRESEASON2014"
    season_4 = "SEASON2014"
    preseason_5 = "PRESEASON2015"
    season_5 = "SEASON2015"
    preseason_6 = "PRESEASON2016"
    season_6 = "SEASON2016"
    preseason_7 = "PRESEASON2017"
    season_7 = "SEASON2017"


class Region(enum.Enum):
    brazil = "br"
    europe_north_east = "eune"
    europe_west = "euw"
    japan = "jp"
    korea = "kr"
    latin_america_north = "lan"
    latin_america_south = "las"
    north_america = "na"
    oceania = "oce"
    pbe = "pbe"
    russia = "ru"
    turkey = "tr"


class Platform(enum.Enum):
    brazil = "BR1"
    europe_north_east = "EUN1"
    europe_west = "EUW1"
    korea = "KR"
    latin_america_north = "LA1"
    latin_america_south = "LA2"
    north_america = "NA1"
    oceania = "OC1"
    russia = "RU"
    turkey = "TR1"


class Map(enum.Enum):
    summoners_rift_summer = 1
    summoners_rift_autumn = 2
    the_proving_grounds = 3
    twisted_treeline_original = 4
    the_crystal_scar = 8
    twisted_treeline = 10
    summoners_rift = 11
    howling_abyss = 12
    butchers_bridge = 14
    cosmic_ruins = 16


class GameMode(enum.Enum):
    aram = "ARAM"
    ascension = "ASCENSION"
    classic = "CLASSIC"
    showdown = "FIRSTBLOOD"
    poro_king = "KINGPORO"
    dominion = "ODIN"
    one_for_all = "ONEFORALL"
    tutorial = "TUTORIAL"
    nexus_siege = "SIEGE"
    assassinate = "ASSASSINATE"
    dark_star = "DARKSTAR"
    arsr = "ARSR"


class GameType(enum.Enum):
    custom = "CUSTOM_GAME"
    tutorial = "TUTORIAL_GAME"
    matched = "MATCHED_GAME"


class SubType(enum.Enum):
    custom = "NONE"
    normal_fives = "NORMAL"
    normal_threes = "NORMAL_3x3"
    dominion = "ODIN_UNRANKED"
    aram = "ARAM_UNRANKED_5x5"
    bot_fives = "BOT"
    bot_threes = "BOT_3x3"
    ranked_solo = "RANKED_SOLO_5x5"
    ranked_threes = "RANKED_TEAM_3x3"
    ranked_fives = "RANKED_TEAM_5x5"
    one_for_all = "ONEFORALL_5x5"
    showdown_solo = "FIRSTBLOOD_1x1"
    showdown_duo = "FIRSTBLOOD_2x2"
    hexakill_summoners_rift = "SR_6x6"
    team_builder = "CAP_5x5"
    urf = "URF"
    bot_urf = "URF_BOT"
    doom_bots = "NIGHTMARE_BOT"
    ascension = "ASCENSION"
    hexakill_twisted_treeline = "HEXAKILL"
    poro_king = "KING_PORO"
    nemesis_draft = "COUNTER_PICK"
    black_market = "BILGEWATER"
    nexus_siege = "SIEGE"
    flex = "RANKED_FLEX_SR"
    flex_threes = "RANKED_FLEX_TT"


class StatSummaryType(enum.Enum):
    normal_fives = "Unranked"
    normal_threes = "Unranked3x3"
    dominion = "OdinUnranked"
    aram = "AramUnranked5x5"
    bot_fives = "CoopVsAI"
    bot_threes = "CoopVsAI3x3"
    ranked_solo = "RankedSolo5x5"
    ranked_threes = "RankedTeam3x3"
    ranked_fives = "RankedTeam5x5"
    one_for_all = "OneForAll5x5"
    showdown_solo = "FirstBlood1x1"
    showdown_duo = "FirstBlood2x2"
    hexakill_summoners_rift = "SummonersRift6x6"
    team_builder = "CAP5x5"
    urf = "URF"
    bot_urf = "URFBots"
    doom_bots = "NightmareBot"
    ascension = "Ascension"
    hexakill_twisted_treeline = "Hexakill"
    poro_king = "KingPoro"
    nemesis_draft = "CounterPick"
    black_market = "Bilgewater"
    nexus_siege = "Siege"
    flex_twisted_treeline = "RankedFlexTT"
    flex_summoners_rift = "RankedFlexSR"
    ranked_premade_threes = "RankedPremade3x3"  # Not listed on game constants documentation page
    ranked_premade_fives = "RankedPremade5x5"   # Not listed on game constants documentation page
    assassinate = "Assassinate"
    Arsr = "Arsr" # All random summoner's rift
    Darkstar = "Darkstar"


class Ascended(enum.Enum):
    player = "CHAMPION_ASCENDED"
    death = "CLEAR_ASCENDED"
    npc = "MINION_ASCENDED"


class Building(enum.Enum):
    inhibitor = "INHIBITOR_BUILDING"
    turret = "TOWER_BUILDING"


class EventType(enum.Enum):
    ascension = "ASCENDED_EVENT"
    building_kill = "BUILDING_KILL"
    point_capture = "CAPTURE_POINT"
    kill = "CHAMPION_KILL"
    elite_monster_kill = "ELITE_MONSTER_KILL"
    item_destruction = "ITEM_DESTROYED"
    item_purchase = "ITEM_PURCHASED"
    item_sale = "ITEM_SOLD"
    item_undo = "ITEM_UNDO"
    summoning = "PORO_KING_SUMMON"
    skill_up = "SKILL_LEVEL_UP"
    ward_kill = "WARD_KILL"
    ward_placement = "WARD_PLACED"


class LaneType(enum.Enum):
    top_lane = "TOP_LANE"
    mid_lane = "MID_LANE"
    bot_lane = "BOT_LANE"


class LevelUp(enum.Enum):
    evolution = "EVOLVE"
    normal = "NORMAL"


class Monster(enum.Enum):
    baron = "BARON_NASHOR"
    dragon = "DRAGON"
    blue = "BLUE_GOLEM"
    red = "RED_LIZARD"
    spider = "VILEMAW"
    rift_herald = "RIFTHERALD"


class Point(enum.Enum):
    windmill = "POINT_C"
    drill = "POINT_D"
    boneyard = "POINT_E"
    quarry = "POINT_A"
    refinery = "POINT_B"


class Turret(enum.Enum):
    outer = "OUTER_TURRET"
    inner = "INNER_TURRET"
    inhibitor = "BASE_TURRET"
    nexus = "NEXUS_TURRET"
    fountain = "FOUNTAIN_TURRET"
    undefined = "UNDEFINED_TURRET"


class Ward(enum.Enum):
    blue_trinket = "BLUE_TRINKET"
    sight = "SIGHT_WARD"
    vision = "VISION_WARD"
    yellow_trinket = "YELLOW_TRINKET"
    upgraded_yellow_trinket = "YELLOW_TRINKET_UPGRADE"
    mushroom = "TEEMO_MUSHROOM"
    undefined = "UNDEFINED"




import cassiopeia.dto.requests
import cassiopeia.type.dto.tournament


def create_tournament_codes(tournament_id, parameters, count=1):
    """
    https://developer.riotgames.com/api/methods#!/1063

    Args:
        tournament_id (int): the tournament ID to generate codes for
        parameters (TournamentCodeParameters): the parameters for the tournament codes
        count (int): the number of codes to generate (max 1000) (default 1)

    Returns:
        list<str>: the created tournament codes
    """
    if count < 1 or count > 1000:
        raise ValueError("Count must be between 1 and 1000")
    request = "https://{server}.api.pvp.net/tournament/public/{version}/code".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"])

    query_params = {
        "tournamentId": tournament_id,
        "count": count
    }
    return cassiopeia.dto.requests.post(request, parameters, query_params, False, True)


def get_tournament_code(tournament_code):
    """
    https://developer.riotgames.com/api/methods#!/1057/3643

    Args:
        tournament_code (str): the tournament code to get information about

    Returns:
        TournamentCode: information about the tournament code
    """
    request = "https://{server}.api.pvp.net/tournament/public/{version}/code/{code}".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"], code=tournament_code)
    return cassiopeia.type.dto.tournament.TournamentCode(cassiopeia.dto.requests.get(request, include_base=False, tournament=True))


def update_tournament_code(tournament_code, parameters):
    """
    https://developer.riotgames.com/api/methods#!/1057/3647

    Args:
        tournament_code (str): the tournament code to update
        parameters (TournamentCodeUpdateParameters): the new parameters for the tournament code
    """
    request = "https://{server}.api.pvp.net/tournament/public/{version}/code/{code}".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"], code=tournament_code)
    cassiopeia.dto.requests.put(request, parameters, include_base=False, tournament=True)


def get_lobby_events(tournament_code):
    """
    https://developer.riotgames.com/api/methods#!/1057/3653

    Args:
        tournament_code (str): the tournament code to get lobby events for

    Returns:
        LobbyEventWrapper: the lobby events for that tournament code
    """
    request = "https://{server}.api.pvp.net/tournament/public/{version}/lobby/events/by-code/{code}".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"], code=tournament_code)
    return cassiopeia.type.dto.tournament.LobbyEventWrapper(cassiopeia.dto.requests.get(request, include_base=False, tournament=True))


def create_tournament_provider(parameters):
    """
    https://developer.riotgames.com/api/methods#!/1057/3646

    Args:
        parameters (ProviderRegistrationParameters): the parameters for the provider

    Returns:
        int: the provider ID
    """
    request = "https://{server}.api.pvp.net/tournament/public/{version}/provider".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"])
    return cassiopeia.dto.requests.post(request, parameters, include_base=False, tournament=True)


def create_tournament(parameters):
    """
    https://developer.riotgames.com/api/methods#!/1057/3649

    Args:
        parameters (TournamentRegistrationParameters): the parameters for the tournament

    Returns:
        int: the tournament ID
    """
    request = "https://{server}.api.pvp.net/tournament/public/{version}/tournament".format(server=cassiopeia.dto.requests.region, version=cassiopeia.dto.requests.api_versions["tournament"])
    return cassiopeia.dto.requests.post(request, parameters, include_base=False, tournament=True)



import cassiopeia.type.dto.common
import cassiopeia.type.core.common


if cassiopeia.type.dto.common.sqlalchemy_imported:
    import sqlalchemy
    import sqlalchemy.orm


@cassiopeia.type.core.common.inheritdocs
class TournamentCodeParameters(cassiopeia.type.dto.common.CassiopeiaParametersDto):
    """
    Args:
        teamSize (int): the team size of the game. Valid values are 1-5.
        spectatorType (str): the spectator type of the game. Valid values are NONE, LOBBYONLY, ALL.
        pickType (str): the pick type of the game. Valid values are BLIND_PICK, DRAFT_MODE, ALL_RANDOM, TOURNAMENT_DRAFT.
        mapType (str): the map type of the game. Valid values are SUMMONERS_RIFT, TWISTED_TREELINE, CRYSTAL_SCAR, and HOWLING_ABYSS.
        allowedSummonerIds (SummonerIdParams): optional list of participants in order to validate the players eligible to join the lobby. NOTE: We currently do not enforce participants at the team level, but rather the aggregate of teamOne and teamTwo. We may add the ability to enforce at the team level in the future.
        metadata (str): optional string that may contain any data in any format, if specified at all. Used to denote any custom information about the game.
    """
    def __init__(self, teamSize, spectatorType, pickType, mapType, allowedSummonerIds=None, metadata=""):
        self.teamSize = teamSize
        self.spectatorType = spectatorType
        self.pickType = pickType
        self.mapType = mapType
        self.allowedSummonerIds = allowedSummonerIds
        self.metadata = metadata


@cassiopeia.type.core.common.inheritdocs
class SummonerIdParams(cassiopeia.type.dto.common.CassiopeiaParametersDto):
    """
    Returns:
        participants: list<int>   the tournament participants
    """
    def __init__(self, participants):
        self.participants = participants


@cassiopeia.type.core.common.inheritdocs
class TournamentCode(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        code (str): the tournament code
        id (int): the tournament code's ID
        lobbyName (str): the lobby name for the tournament code game
        map (str): the game map for the tournament code game
        metaData (str): the metadata for tournament code
        participants (list<int>): the IDs of the summoners participating in the tournament
        password (str): the password for the tournament code game
        pickType (str): the pick mode for tournament code game
        providerId (int): the provider's ID
        region (str): the tournament code's region (Legal values: BR, EUNE, EUW, JP, KR, LAN, LAS, NA, OCE, PBE, RU, TR)
        spectators (str): the spectator mode for the tournament code game
        teamSize (int): the team size for the tournament code game
        tournamentId (int): the tournament's ID
    """
    def __init__(self, dictionary):
        self.code = dictionary.get("code", "")
        self.id = dictionary.get("id", 0)
        self.lobbyName = dictionary.get("lobbyName", "")
        self.map = dictionary.get("map", "")
        self.metaData = dictionary.get("metaData", "")
        self.participants = dictionary.get("participants", [])
        self.password = dictionary.get("password", "")
        self.pickType = dictionary.get("pickType", "")
        self.providerId = dictionary.get("providerId", 0)
        self.region = dictionary.get("region", "")
        self.spectators = dictionary.get("spectators", "")
        self.teamSize = dictionary.get("teamSize", 0)
        self.tournamentId = dictionary.get("tournamentId", 0)


@cassiopeia.type.core.common.inheritdocs
class TournamentCodeUpdateParameters(cassiopeia.type.dto.common.CassiopeiaParametersDto):
    """
    Args:
        allowedParticipants (str): comma separated list of summoner Ids
        spectatorType (str): the spectator type (Legal values: NONE, LOBBYONLY, ALL)
        pickType (str): the pick type (Legal values: BLIND_PICK, DRAFT_MODE, ALL_RANDOM, TOURNAMENT_DRAFT)
        mapType (str): the map type (Legal values: SUMMONERS_RIFT, CRYSTAL_SCAR, TWISTED_TREELINE, HOWLING_ABYSS)
    """
    def __init__(self, allowedParticipants="", spectatorType="", pickType="", mapType=""):
        self.allowedParticipants = allowedParticipants
        self.spectatorType = spectatorType
        self.pickType = pickType
        self.mapType = mapType


@cassiopeia.type.core.common.inheritdocs
class LobbyEventWrapper(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        eventList (list<LobbyEvent>): the list of events
    """
    def __init__(self, dictionary):
        self.eventList = [(LobbyEvent(event) if not isinstance(event, LobbyEvent) else event) for event in dictionary.get("eventList", []) if event]


@cassiopeia.type.core.common.inheritdocs
class LobbyEvent(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        eventType (str): the type of event that was triggered
        summonerId (str): the summoner that triggered the event
        timestamp (str): timestamp from the event
    """
    def __init__(self, dictionary):
        self.eventType = dictionary.get("eventType", "")
        self.summonerId = dictionary.get("summonerId", "")
        self.timestamp = dictionary.get("timestamp", "")


@cassiopeia.type.core.common.inheritdocs
class ProviderRegistrationParameters(cassiopeia.type.dto.common.CassiopeiaParametersDto):
    """
    Args:
        region (str): the region in which the provider will be running tournaments (Legal values: BR, EUNE, EUW, JP, KR, LAN, LAS, NA, OCE, PBE, RU, TR)
        url (str): the provider's callback URL to which tournament game results in this region should be posted. The URL must be well-formed, use the http or https protocol, and use the default port for the protocol (http URLs must use port 80, https URLs must use port 443).
    """
    def __init__(self, region, url):
        self.region = region
        self.url = url


@cassiopeia.type.core.common.inheritdocs
class TournamentRegistrationParameters(cassiopeia.type.dto.common.CassiopeiaParametersDto):
    """
    Args:
        providerId (int): the provider ID to specify the regional registered provider data to associate this tournament
        name (str): the optional name of the tournament
    """
    def __init__(self, providerId, name=""):
        self.providerId = providerId
        self.name = name


###############################
# Dynamic SQLAlchemy bindings #
###############################
def _sa_bind_tournament_code():
    global TournamentCode

    @cassiopeia.type.core.common.inheritdocs
    class TournamentCode(TournamentCode, cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "TournamentCode"
        code = sqlalchemy.Column(sqlalchemy.String(50))
        id = sqlalchemy.Column(sqlalchemy.BigInteger, primary_key=True)
        lobbyName = sqlalchemy.Column(sqlalchemy.String(50))
        map = sqlalchemy.Column(sqlalchemy.String(30))
        metaData = sqlalchemy.Column(sqlalchemy.Text)
        participants = sqlalchemy.Column(cassiopeia.type.dto.common.JSONEncoded)
        password = sqlalchemy.Column(sqlalchemy.String(30))
        pickType = sqlalchemy.Column(sqlalchemy.String(30))
        providerId = sqlalchemy.Column(sqlalchemy.Integer)
        region = sqlalchemy.Column(sqlalchemy.String(30))
        spectators = sqlalchemy.Column(sqlalchemy.String(30))
        teamSize = sqlalchemy.Column(sqlalchemy.Integer)
        tournamentId = sqlalchemy.Column(sqlalchemy.BigInteger)


def _sa_bind_lobby_event():
    global LobbyEvent

    @cassiopeia.type.core.common.inheritdocs
    class LobbyEvent(LobbyEvent, cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "LobbyEvent"
        _id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
        eventType = sqlalchemy.Column(sqlalchemy.String(50))
        summonerId = sqlalchemy.Column(sqlalchemy.String(30))
        timestamp = sqlalchemy.Column(sqlalchemy.String(50))


def _sa_bind_all():
    _sa_bind_tournament_code()
    _sa_bind_lobby_event()



import enum

import cassiopeia.riotapi
import cassiopeia.type.core.common
import cassiopeia.type.dto.tournament


class MapType(enum.Enum):
    summoners_rift = "SUMMONERS_RIFT"
    twisted_treeline = "TWISTED_TREELINE"
    crystal_scar = "CRYSTAL_SCAR"
    howling_abyss = "HOWLING_ABYSS"


class PickType(enum.Enum):
    blind = "BLIND_PICK"
    draft = "DRAFT_MODE"
    random = "ALL_RANDOM"
    tournament_draft = "TOURNAMENT_DRAFT"


class SpectatorType(enum.Enum):
    none = "NONE"
    lobby = "LOBBYONLY"
    all = "ALL"


class TournamentRegion(enum.Enum):
    brazil = "BR"
    europe_north_east = "EUNE"
    europe_west = "EUW"
    japan = "JP"
    korea = "KR"
    latin_america_north = "LAN"
    latin_america_south = "LAS"
    north_america = "NA"
    oceania = "OCE"
    pbe = "PBE"
    russia = "RU"
    turkey = "TR"


@cassiopeia.type.core.common.inheritdocs
class TournamentCode(cassiopeia.type.core.common.CassiopeiaObject):
    dto_type = cassiopeia.type.dto.tournament.TournamentCode

    def __str__(self):
        return "{name} ({code})".format(name=self.name, code=self.code)

    @property
    def code(self):
        """
        Returns:
            str: the tournament code
        """
        return self.data.code

    @property
    def id(self):
        """
        Returns:
            int: the tournament code's ID
        """
        return self.data.id

    @property
    def name(self):
        """
        Returns:
            str: the lobby name
        """
        return self.data.lobbyName

    @property
    def map(self):
        """
        Returns:
            str: the map for the game
        """
        return MapType(self.data.map) if self.data.map else None

    @property
    def meta_data(self):
        """
        Returns:
            str: the metadata for the game
        """
        return self.data.metaData

    @property
    def participants(self):
        """
        Returns:
            list<Summoner>: the summoners participating in the tournament
        """
        return cassiopeia.riotapi.get_summoners_by_id(self.data.participants) if self.data.participants else []

    @property
    def password(self):
        """
        Returns:
            str: the password for the lobby
        """
        return self.data.password

    @property
    def pick_type(self):
        """
        Returns:
            PickType: the pick mode for the game
        """
        return PickType(self.data.pickType) if self.data.pickType else None

    @property
    def provider_id(self):
        """
        Returns:
            int: the provider's ID
        """
        return self.data.providerId

    @property
    def region(self):
        """
        Returns:
            TournamentRegion: the tournament's region
        """
        return TournamentRegion(self.data.region) if self.data.region else None

    @property
    def spectator_type(self):
        """
        Returns:
            SpectatorType: the spectator mode for the game
        """
        return SpectatorType(self.data.spectators) if self.data.spectators else None

    @property
    def team_size(self):
        """
        Returns:
            int: the team size for the game
        """
        return self.data.teamSize

    @property
    def tournament_id(self):
        """
        Returns:
            int: the tournament's ID
        """
        return self.data.tournamentId


@cassiopeia.type.core.common.inheritdocs
class LobbyEvent(cassiopeia.type.core.common.CassiopeiaObject):
    dto_type = cassiopeia.type.dto.tournament.LobbyEvent

    def __str__(self):
        return "{type} ({summoner})".format(type=self.type, summoner=self.summoner)

    @property
    def type(self):
        """
        Returns:
            str: the type of the event
        """
        return self.data.eventType

    @property
    def summoner(self):
        """
        Returns:
            Summoner: the summoner that triggered the event
        """
        return cassiopeia.riotapi.get_summoner_by_id(int(self.data.summonerId)) if self.data.summonerId else None

    @property
    def timestamp(self):
        """
        Returns:
            str: the time that the event occurred
        """
        return self.data.timestamp


###############################
# Dynamic SQLAlchemy bindings #
###############################
def _sa_rebind_all():
    TournamentCode.dto_type = cassiopeia.type.dto.tournament.TournamentCode
    LobbyEvent.dto_type = cassiopeia.type.dto.tournament.LobbyEvent



import cassiopeia.type.dto.common
import cassiopeia.type.core.common


if cassiopeia.type.dto.common.sqlalchemy_imported:
    import sqlalchemy
    import sqlalchemy.orm


class DataStore(object):
    """
    A place to store data. Used for caching/storing data from API calls
    """

    def has_all(self, class_):
        """
        A place to store data. Used for caching/storing data from API calls
        """
        pass

    def get_all(self, class_):
        """
        Checks if the data store has all the values for a type there can be (as reported by the user)

        Args:
            class_ (type): the class to do the check for

        Returns:
            bool: whether all the values are stored
        """
        pass

    def iterate(self, class_):
        """
        Gets all currently stored values for a type

        Args:
            class_ (type): the class to get values for

        Returns:
            list<class_>: all stored values for the type
        """
        pass

    def get(self, class_, keys, key_field):
        """
        Gets an iterator over all currently stored values for a type

        Args:
            class_ (type): the class to get values for

        Returns:
            iterator<class_>: and iterator over all stored values for the type
        """
        pass

    def store(self, objs, keys, complete_sets=[]):
        """
        Gets objects from the data store

        Args:
            class_ (type): the class to get values for
            keys (any | list<any>): the keys that should be used to find the desired values
            key_field (str): the name of the attribute that the key(s) reference

        Returns:
            class_ | list<class_>: the values from storage - None will replace any value that couldn't be found
        """
        pass


#######################
# Void Store resouces #
#######################
@cassiopeia.type.core.common.inheritdocs
class VoidDataStore(DataStore):
    """
    Stores objects in the data store

    Args:
        objs (any): the objects to store
        keys (any | list<any>): the keys to store those values with
        complete_sets (list<type>): include any types for which it should be marked that all possible values are stored
    """

    def has_all(self, class_):
        return False

    def get_all(self, class_):
        return []

    def iterate(self, class_):
        return iter([])

    def get(self, class_, keys, key_field):
        if isinstance(keys, list):
            return [None for _ in range(len(keys))]
        return None

    def store(self, objs, keys, complete_sets=[]):
        pass


#############################
# In-memory Cache resources #
#############################
@cassiopeia.type.core.common.inheritdocs
class Cache(DataStore):
    """
    Stores objects in the data store

    Args:
        objs (any): the objects to store
        keys (any | list<any>): the keys to store those values with
        complete_sets (list<type>): include any types for which it should be marked that all possible values are stored
    """

    def __init__(self):
        self._cache = {}
        self._has_all = {}

    def has_all(self, class_):
        try:
            return self._has_all[class_]
        except KeyError:
            return False

    def get_all(self, class_):
        results = []
        try:
            for obj in self._cache[class_].items():
                results.append(obj[1])
        except KeyError:
            pass

        return results

    def iterate(self, class_):
        try:
            iter(self._cache[class_].values())
        except KeyError:
            return iter([])

    def get(self, class_, keys, key_field):
        if class_ not in self._cache:
            if not isinstance(keys, list):
                return None
            else:
                results = []
                for _ in range(len(keys)):
                    results.append(None)
                return results

        if not isinstance(keys, list):
            try:
                return self._cache[class_][keys]
            except KeyError:
                return None
        else:
            results = []
            for key in keys:
                try:
                    results.append(self._cache[class_][key])
                except KeyError:
                    results.append(None)
            return results

    def store(self, objs, keys, complete_sets=[]):
        is_list = isinstance(objs, list)
        if is_list != isinstance(keys, list):
            raise ValueError("Object(s) and Key(s) must both be lists or both be non-lists")

        if not is_list:
            class_ = type(objs)
            if class_ not in self._cache:
                self._cache[class_] = {}
            self._cache[class_][keys] = objs
        else:
            if len(objs) != len(keys):
                raise ValueError("Objects and Keys must be the same length")

            for i in range(len(objs)):
                class_ = type(objs[i])
                if class_ not in self._cache:
                    self._cache[class_] = {}
                self._cache[class_][keys[i]] = objs[i]

        if complete_sets:
            for class_ in complete_sets:
                self._has_all[class_] = True


########################
# SQLAlchemy resources #
########################
if cassiopeia.type.dto.common.sqlalchemy_imported:
    class HasAllStatus(cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "HasAll"
        class_ = sqlalchemy.Column(sqlalchemy.String(50), primary_key=True)
        have_all = sqlalchemy.Column(sqlalchemy.Boolean)

        @staticmethod
        def get_name(class_):
            return "{module}.{name}".format(module=class_.dto_type.__module__, name=class_.dto_type.__name__)

        def __init__(self, class_, have_all=True):
            self.class_ = HasAllStatus.get_name(class_)
            self.have_all = have_all

    @cassiopeia.type.core.common.inheritdocs
    class SQLAlchemyDB(DataStore):
        class Iterator(object):
            def __init__(self, class_, result):
                self.class_ = class_
                self.result = result

            def __next__(self):
                try:
                    val = self.class_(self.result[self.index])
                    self.index += 1
                    return val
                except IndexError:
                    raise StopIteration

            def __iter__(self):
                self.index = 0
                return self

        def __init__(self, flavor, host, database, username, password):
            """
            Stores objects in the data store

            Args:
                objs (any): the objects to store
                keys (any | list<any>): the keys to store those values with
                complete_sets (list<type>): include any types for which it should be marked that all possible values are stored
            """
            _sa_bind_typesystem()
            if flavor == 'sqlite':
                self.db = sqlalchemy.create_engine("{flavor}://{database}".format(flavor=flavor, database=database))
            else:
                self.db = sqlalchemy.create_engine("{flavor}://{username}:{password}@{host}/{database}".format(flavor=flavor, host=host, database=database, username=username, password=password))
            cassiopeia.type.dto.common.BaseDB.metadata.create_all(self.db)
            self.session = sqlalchemy.orm.sessionmaker(bind=self.db)()

        def has_all(self, class_):
            class_name = HasAllStatus.get_name(class_)
            has_all = self.session.query(HasAllStatus).filter(HasAllStatus.class_ == class_name).first()
            return has_all.have_all if has_all else False

        def get_all(self, class_):
            return [class_(dto) for dto in self.session.query(class_.dto_type).all()]

        def iterate(self, class_):
            return SQLAlchemyDB.Iterator(class_, self.session.query(class_.dto_type).all())

        def get(self, class_, keys, key_field):
            if not isinstance(keys, list):
                val = self.session.query(class_.dto_type).filter(getattr(class_.dto_type, key_field) == keys).first()
                return class_(val) if val else None
            else:
                from_db = self.session.query(class_.dto_type).filter(getattr(class_.dto_type, key_field).in_(keys)).all()
                from_db = {getattr(x, key_field): x for x in from_db}

                results = []
                for key in keys:
                    try:
                        val = from_db[key]
                        results.append(class_(val) if val else None)
                    except KeyError:
                        results.append(None)
                return results

        def store(self, objs, keys=None, complete_sets=[]):
            if not isinstance(objs, list):
                class_ = objs.data.__class__
                p_key = class_.__mapper__.primary_key[0].name.split("\\.")[-1]
                val = self.session.query(class_).filter(getattr(class_, p_key) == getattr(objs.data, p_key)).first()
                if not val:
                    self.session.add(objs.data)
            else:
                class_ = objs[0].data.__class__
                p_key = class_.__mapper__.primary_key[0].name.split("\\.")[-1]
                to_store = {getattr(obj.data, p_key): obj.data for obj in objs}

                for obj in self.session.query(class_).filter(getattr(class_, p_key).in_(to_store.keys())).all():
                    del to_store[getattr(obj, p_key)]

                self.session.add_all(to_store.values())

            if complete_sets:
                classes = {HasAllStatus.get_name(class_): HasAllStatus(class_) for class_ in complete_sets}
                for obj in self.session.query(HasAllStatus).filter(HasAllStatus.class_.in_(classes.keys())).all():
                    if not obj.have_all:
                        obj.have_all = True
                    del classes[obj.class_]
                self.session.add_all(classes.values())

            self.session.commit()

        def close(self):
            """
            A mock cache that doesn't actually store anything
            """
            self.session.close()
            self.db.dispose()


__sa_bound = False


def _sa_bind_typesystem():
    """
    A mock cache that doesn't actually store anything
    """
    global __sa_bound
    if __sa_bound:
        return

    import cassiopeia.type.dto.champion
    import cassiopeia.type.dto.championmastery
    import cassiopeia.type.dto.currentgame
    import cassiopeia.type.dto.featuredgames
    import cassiopeia.type.dto.game
    import cassiopeia.type.dto.league
    import cassiopeia.type.dto.match
    import cassiopeia.type.dto.matchlist
    import cassiopeia.type.dto.staticdata
    import cassiopeia.type.dto.stats
    import cassiopeia.type.dto.status
    import cassiopeia.type.dto.summoner
    import cassiopeia.type.dto.team
    import cassiopeia.type.dto.tournament
    cassiopeia.type.dto.champion._sa_bind_all()
    cassiopeia.type.dto.championmastery._sa_bind_all()
    cassiopeia.type.dto.currentgame._sa_bind_all()
    cassiopeia.type.dto.featuredgames._sa_bind_all()
    cassiopeia.type.dto.game._sa_bind_all()
    cassiopeia.type.dto.league._sa_bind_all()
    cassiopeia.type.dto.match._sa_bind_all()
    cassiopeia.type.dto.matchlist._sa_bind_all()
    cassiopeia.type.dto.staticdata._sa_bind_all()
    cassiopeia.type.dto.stats._sa_bind_all()
    cassiopeia.type.dto.status._sa_bind_all()
    cassiopeia.type.dto.summoner._sa_bind_all()
    cassiopeia.type.dto.team._sa_bind_all()
    cassiopeia.type.dto.tournament._sa_bind_all()

    import cassiopeia.type.core.champion
    import cassiopeia.type.core.championmastery
    import cassiopeia.type.core.currentgame
    import cassiopeia.type.core.featuredgames
    import cassiopeia.type.core.game
    import cassiopeia.type.core.league
    import cassiopeia.type.core.match
    import cassiopeia.type.core.matchlist
    import cassiopeia.type.core.staticdata
    import cassiopeia.type.core.stats
    import cassiopeia.type.core.status
    import cassiopeia.type.core.summoner
    import cassiopeia.type.core.team
    import cassiopeia.type.core.tournament
    cassiopeia.type.core.champion._sa_rebind_all()
    cassiopeia.type.core.championmastery._sa_rebind_all()
    cassiopeia.type.core.currentgame._sa_rebind_all()
    cassiopeia.type.core.featuredgames._sa_rebind_all()
    cassiopeia.type.core.game._sa_rebind_all()
    cassiopeia.type.core.league._sa_rebind_all()
    cassiopeia.type.core.match._sa_rebind_all()
    cassiopeia.type.core.matchlist._sa_rebind_all()
    cassiopeia.type.core.staticdata._sa_rebind_all()
    cassiopeia.type.core.stats._sa_rebind_all()
    cassiopeia.type.core.status._sa_rebind_all()
    cassiopeia.type.core.summoner._sa_rebind_all()
    cassiopeia.type.core.team._sa_rebind_all()
    cassiopeia.type.core.tournament._sa_rebind_all()

    __sa_bound = True



from cassiopeia import riotapi
from .. import int_test_handler


def test_all():
    print("core/featuredgamesapi tests...")
    test_featured_games()


def test_featured_games():
    int_test_handler.test_result(riotapi.get_featured_games())



#!/usr/bin/env python

import sys

from setuptools import setup, find_packages


install_requires = [
    "sqlalchemy"
]

# Install python 2.7 dependencies
if sys.version_info.major == 2:
    install_requires += [
        "future==0.15.2",
        "enum34==1.1.1",
    ]

setup(
    name="cassiopeia",
    version="0.1.4",
    author="Rob Rua",
    author_email="robrua@alumni.cmu.edu",
    url="https://github.com/meraki-analytics/cassiopeia",
    description="Riot Games Developer API Wrapper (3rd Party)",
    keywords=["LoL", "League of Legends", "Riot Games", "API", "REST"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Games/Entertainment",
        "Topic :: Games/Entertainment :: Real Time Strategy",
        "Topic :: Games/Entertainment :: Role-Playing",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license="MIT",
    packages=find_packages(),
    zip_safe=True,
    install_requires=install_requires
)



import cassiopeia.dto.tournamentapi
import cassiopeia.type.core.tournament
import cassiopeia.type.dto.tournament


def create_tournament_codes(tournament_id, team_size, spectator_type, pick_type, map_type, allowed_summoners=[], meta_data="", count=1):
    """
    Creates tournament codes for a tournament

    Args:
        tournament_id (int): the tournament ID to generate codes for
        team_size (int): the team size for the tournament (1-5)
        spectator_type (str | SpectatorType): the spectator availability for the tournament
        pick_type (str | PickType): the pick type for the tournament
        map_type (str | MapType): the map the tournament is played on
        allowed_summoners (list<Summoner>): the summoners who are allowed to participate in the tournament (default [])
        meta_data (str | object): meta data to be included with the tournament. Any non-string value will be cast to a string. (default "")
        count (int): the number of codes to generate (max 1000) (default 1)

    Returns:
        list<str>: the created tournament codes
    """
    if team_size < 1 or team_size > 5:
        raise ValueError("Team Size must be between 1 and 5!")
    if not isinstance(spectator_type, cassiopeia.type.core.tournament.SpectatorType):
        spectator_type = cassiopeia.type.core.tournament.SpectatorType(spectator_type.upper())
    if not isinstance(pick_type, cassiopeia.type.core.tournament.PickType):
        pick_type = cassiopeia.type.core.tournament.PickType(pick_type.upper())
    if not isinstance(map_type, cassiopeia.type.core.tournament.MapType):
        map_type = cassiopeia.type.core.tournament.MapType(map_type.upper())

    summoners = cassiopeia.type.dto.tournament.SummonerIdParams([summoner.id for summoner in allowed_summoners]) if allowed_summoners else None
    parameters = cassiopeia.type.dto.tournament.TournamentCodeParameters(team_size, spectator_type.value, pick_type.value, map_type.value, summoners, str(meta_data))

    return cassiopeia.dto.tournamentapi.create_tournament_codes(tournament_id, parameters, count)


def get_tournament_code(tournament_code):
    """
    Gets information about the tournament code

    Args:
        tournament_code (str): the tournament code

    Returns:
        TournamentCode: the tournament code information
    """
    return cassiopeia.type.core.tournament.TournamentCode(cassiopeia.dto.tournamentapi.get_tournament_code(tournament_code))


def update_tournament_code(tournament_code, allowed_summoners=[], spectator_type=None, pick_type=None, map_type=None):
    """
    Updates a tournament code

Args:
        tournament_code (str | TournamentCode): the tournament code to update
        allowed_summoners (list<Summoner>): the summoners who are allowed to participate in the tournament (default [])
        spectator_type (str | SpectatorType): the spectator availability for the tournament (default None)
        pick_type (str | PickType): the pick type for the tournament (default None)
        map_type (str | MapType): the map the tournament is played on (default None)
    """
    if isinstance(tournament_code, cassiopeia.type.core.tournament.TournamentCode):
        tournament_code = tournament_code.code
    if spectator_type and not isinstance(spectator_type, cassiopeia.type.core.tournament.SpectatorType):
        spectator_type = cassiopeia.type.core.tournament.SpectatorType(spectator_type.upper())
    if pick_type and not isinstance(pick_type, cassiopeia.type.core.tournament.PickType):
        pick_type = cassiopeia.type.core.tournament.PickType(pick_type.upper())
    if map_type and not isinstance(map_type, cassiopeia.type.core.tournament.MapType):
        map_type = cassiopeia.type.core.tournament.MapType(map_type.upper())
    summoners = ",".join([str(summoner.id) for summoner in allowed_summoners])
    parameters = cassiopeia.type.dto.tournament.TournamentCodeUpdateParameters(summoners, spectator_type.value if spectator_type else "", pick_type.value if pick_type else "", map_type.value if map_type else "")
    cassiopeia.dto.tournamentapi.update_tournament_code(tournament_code, parameters)


def get_lobby_events(tournament_code):
    """
    Gets the lobby events that have occurred for the tournament code

    Args:
        tournament_code (str | TournamentCode): the tournament code to get lobby events for

    Returns:
        list<LobbyEvent>: the lobby events for that tournament code
    """
    if isinstance(tournament_code, cassiopeia.type.core.tournament.TournamentCode):
        tournament_code = tournament_code.code
    return [cassiopeia.type.core.tournament.LobbyEvent(event) for event in cassiopeia.dto.tournamentapi.get_lobby_events(tournament_code).eventList]


def create_tournament_provider(region, url):
    """
    Creates a tournament provider

    Args:
        region (str | TournamentRegion): the region in which the provider will be running tournaments
        url (str): the provider's callback URL to which tournament game results in this region should be posted. The URL must be well-formed, use the http or https protocol, and use the default port for the protocol (http URLs must use port 80, https URLs must use port 443).

    Returns:
        int: the tournament provider ID
    """
    if isinstance(region, cassiopeia.type.core.common.Region):
        region = cassiopeia.type.core.tournament.TournamentRegion(region.value.upper())
    elif not isinstance(region, cassiopeia.type.core.tournament.TournamentRegion):
        region = cassiopeia.type.core.tournament.TournamentRegion(region.upper())

    parameters = cassiopeia.type.dto.tournament.ProviderRegistrationParameters(region.value, url)
    return cassiopeia.dto.tournamentapi.create_tournament_provider(parameters)


def create_tournament(provider_id, name=""):
    """
    Creates a tournament

    Args:
        provider_id (int): the provider ID to specify the regional registered provider data to associate this tournament
        name (str): the optional name of the tournament (default "")

    Returns:
        int: the tournament ID
    """
    parameters = cassiopeia.type.dto.tournament.TournamentRegistrationParameters(provider_id, name)
    return cassiopeia.dto.tournamentapi.create_tournament(parameters)



import cassiopeia.riotapi
import cassiopeia.dto.championapi
import cassiopeia.type.core.champion


def get_champion_statuses(free_to_play=False):
    """
    Gets the statuses for all champions (whether they are disabled, etc.)

    Args:
        free_to_play (bool): whether to only return free champions (default False)

    Returns:
        dict<Champion, ChampionStatus>: the statuses for all the champions
    """
    statuses = cassiopeia.dto.championapi.get_champion_statuses(free_to_play)

    # Always load champions since we'll be using them here
    cassiopeia.riotapi.get_champions()
    return {cassiopeia.riotapi.get_champion_by_id(champ.id): cassiopeia.type.core.champion.ChampionStatus(champ) for champ in statuses.champions}


def get_champion_status(champion):
    """
    Gets the status for a champion (whether they are disabled, etc.)

    Args:
        champion (Champion): the champion to get the status of

    Returns:
        ChampionStatus: the champion's status
    """
    status = cassiopeia.dto.championapi.get_champion_status(champion.id)
    return cassiopeia.type.core.champion.ChampionStatus(status)



import cassiopeia.dto.requests
import cassiopeia.type.dto.matchlist


def get_match_list(summoner_id, num_matches=0, begin_index=0, begin_time=0, end_time=0, champion_ids=None, ranked_queues=None, seasons=None):
    """
    https://developer.riotgames.com/api/methods#!/1013/3439

    Args:
        summoner_id (int): the ID of the summoner to get the match history for
        num_matches (int): the maximum number of matches to retrieve. 0 will get as many as possible. (default 0)
        begin_index (int): the game index to start from (default 0)
        begin_time (int): the begin time to use for fetching games specified as epoch milliseconds (default 0)
        end_time (int): the end time to use for fetching games specified as epoch milliseconds (default 0)
        champion_ids (int | list<int>): the champion ID(s) to limit the results to (default None)
        ranked_queues (str | list<str>): the ranked queue(s) to limit the results to ("RANKED_SOLO_5x5", "RANKED_TEAM_3x3", "RANKED_TEAM_5x5") (default None)
        seasons (str | list<str>): the season(s) to limit the results to ("PRESEASON3", "SEASON3", "PRESEASON2014", "SEASON2014", "PRESEASON2015", "SEASON2015", "PRESEASON2016", "SEASON2016") (default None)

    Returns:
        MatchList: the summoner's match history
    """
    request = "{version}/matchlist/by-summoner/{summoner_id}".format(version=cassiopeia.dto.requests.api_versions["matchlist"], summoner_id=summoner_id)

    params = {}
    if num_matches:
        params["endIndex"] = begin_index + num_matches
    if begin_index or num_matches:
        params["beginIndex"] = begin_index
    if begin_time:
        params["beginTime"] = begin_time
    if end_time:
        params["endTime"] = end_time
    if champion_ids:
        params["championIds"] = ",".join(champion_ids) if isinstance(champion_ids, list) else str(champion_ids)
    if ranked_queues:
        params["rankedQueues"] = ",".join(ranked_queues) if isinstance(ranked_queues, list) else str(ranked_queues)
    if seasons:
        params["seasons"] = ",".join(seasons) if isinstance(seasons, list) else str(seasons)

    return cassiopeia.type.dto.matchlist.MatchList(cassiopeia.dto.requests.get(request, params))



import cassiopeia.type.dto.common
import cassiopeia.type.core.common


if cassiopeia.type.dto.common.sqlalchemy_imported:
    import sqlalchemy
    import sqlalchemy.orm


@cassiopeia.type.core.common.inheritdocs
class RawStats(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        assists (int): number of assists
        barracksKilled (int): number of enemy inhibitors killed
        championsKilled (int): number of champions killed
        combatPlayerScore (int): the combat player score
        consumablesPurchased (int): number of consumables purchased
        damageDealtPlayer (int): total damage dealt
        doubleKills (int): number of double kills
        firstBlood (int): first blood
        gold (int): amount of gold
        goldEarned (int): total gold earned
        goldSpent (int): total gold spent
        item0 (int): ID of item 0
        item1 (int): ID of item 1
        item2 (int): ID of item 2
        item3 (int): ID of item 3
        item4 (int): ID of item 4
        item5 (int): ID of item 5
        item6 (int): ID of item 6
        itemsPurchased (int): number of items purchased
        killingSprees (int): number of killing sprees
        largestCriticalStrike (int): largest critical strike
        largestKillingSpree (int): largest killing spree
        largestMultiKill (int): largest multi kill
        legendaryItemsCreated (int): number of tier 3 items built
        level (int): level
        magicDamageDealtPlayer (int): total magic damage dealt
        magicDamageDealtToChampions (int): total magic damage dealt to champions
        magicDamageTaken (int): total magic damage taken
        minionsDenied (int): total minions denied
        minionsKilled (int): total minions killed
        neutralMinionsKilled (int): total neutral minions killed
        neutralMinionsKilledEnemyJungle (int): neutral minions killed in enemy jungle
        neutralMinionsKilledYourJungle (int): neutral minions killed in own jungle
        nexusKilled (bool): flag specifying if the summoner got the killing blow on the nexus
        nodeCapture (int): number of nodes captured
        nodeCaptureAssist (int): number of node capture assists
        nodeNeutralize (int): number of nodes neutralized
        nodeNeutralizeAssist (int): number of node neutralization assists
        numDeaths (int): number of deaths
        numItemsBought (int): number of items bought
        objectivePlayerScore (int): objective player score
        pentaKills (int): number of penta kills
        physicalDamageDealtPlayer (int): total physical damage dealt
        physicalDamageDealtToChampions (int): total physical damage dealt to champions
        physicalDamageTaken (int): total physical damage taken
        playerPosition (int): player position
        playerRole (int): player role
        quadraKills (int): number of quadra kills
        sightWardsBought (int): number of sight wards bought
        spell1Cast (int): number of times first champion spell was cast
        spell2Cast (int): number of times second champion spell was cast
        spell3Cast (int): number of times third champion spell was cast
        spell4Cast (int): number of times fourth champion spell was cast
        summonSpell1Cast (int): number of times summoner spell 1 was cast
        summonSpell2Cast (int): number of times summoner spell 2 was cast
        superMonsterKilled (int): number of super monsters killed
        team (int): team
        teamObjective (int): team objectives
        timePlayed (int): time played
        totalDamageDealt (int): total damage dealt
        totalDamageDealtToChampions (int): total damage dealt to champions
        totalDamageTaken (int): total damage taken
        totalHeal (int): total healing done
        totalPlayerScore (int): total player score
        totalScoreRank (int): total score rank
        totalTimeCrowdControlDealt (int): total crowd control time dealt
        totalUnitsHealed (int): number of units healed
        tripleKills (int): number of triple kills
        trueDamageDealtPlayer (int): total true damage dealt
        trueDamageDealtToChampions (int): total true damage dealt to champions
        trueDamageTaken (int): total true damage taken
        turretsKilled (int): number of turrets killed
        unrealKills (int): number of unreal kills
        victoryPointTotal (int): total victory points
        visionWardsBought (int): number of vision wards bought
        wardKilled (int): number of wards killed
        wardPlaced (int): number of wards placed
        win (bool): flag specifying whether or not this game was won
    """
    def __init__(self, dictionary):
        self.assists = dictionary.get("assists", 0)
        self.barracksKilled = dictionary.get("barracksKilled", 0)
        self.championsKilled = dictionary.get("championsKilled", 0)
        self.combatPlayerScore = dictionary.get("combatPlayerScore", 0)
        self.consumablesPurchased = dictionary.get("consumablesPurchased", 0)
        self.damageDealtPlayer = dictionary.get("damageDealtPlayer", 0)
        self.doubleKills = dictionary.get("doubleKills", 0)
        self.firstBlood = dictionary.get("firstBlood", 0)
        self.gold = dictionary.get("gold", 0)
        self.goldEarned = dictionary.get("goldEarned", 0)
        self.goldSpent = dictionary.get("goldSpent", 0)
        self.item0 = dictionary.get("item0", 0)
        self.item1 = dictionary.get("item1", 0)
        self.item2 = dictionary.get("item2", 0)
        self.item3 = dictionary.get("item3", 0)
        self.item4 = dictionary.get("item4", 0)
        self.item5 = dictionary.get("item5", 0)
        self.item6 = dictionary.get("item6", 0)
        self.itemsPurchased = dictionary.get("itemsPurchased", 0)
        self.killingSprees = dictionary.get("killingSprees", 0)
        self.largestCriticalStrike = dictionary.get("largestCriticalStrike", 0)
        self.largestKillingSpree = dictionary.get("largestKillingSpree", 0)
        self.largestMultiKill = dictionary.get("largestMultiKill", 0)
        self.legendaryItemsCreated = dictionary.get("legendaryItemsCreated", 0)
        self.level = dictionary.get("level", 0)
        self.magicDamageDealtPlayer = dictionary.get("magicDamageDealtPlayer", 0)
        self.magicDamageDealtToChampions = dictionary.get("magicDamageDealtToChampions", 0)
        self.magicDamageTaken = dictionary.get("magicDamageTaken", 0)
        self.minionsDenied = dictionary.get("minionsDenied", 0)
        self.minionsKilled = dictionary.get("minionsKilled", 0)
        self.neutralMinionsKilled = dictionary.get("neutralMinionsKilled", 0)
        self.neutralMinionsKilledEnemyJungle = dictionary.get("neutralMinionsKilledEnemyJungle", 0)
        self.neutralMinionsKilledYourJungle = dictionary.get("neutralMinionsKilledYourJungle", 0)
        self.nexusKilled = dictionary.get("nexusKilled", False)
        self.nodeCapture = dictionary.get("nodeCapture", 0)
        self.nodeCaptureAssist = dictionary.get("nodeCaptureAssist", 0)
        self.nodeNeutralize = dictionary.get("nodeNeutralize", 0)
        self.nodeNeutralizeAssist = dictionary.get("nodeNeutralizeAssist", 0)
        self.numDeaths = dictionary.get("numDeaths", 0)
        self.numItemsBought = dictionary.get("numItemsBought", 0)
        self.objectivePlayerScore = dictionary.get("objectivePlayerScore", 0)
        self.pentaKills = dictionary.get("pentaKills", 0)
        self.physicalDamageDealtPlayer = dictionary.get("physicalDamageDealtPlayer", 0)
        self.physicalDamageDealtToChampions = dictionary.get("physicalDamageDealtToChampions", 0)
        self.physicalDamageTaken = dictionary.get("physicalDamageTaken", 0)
        self.playerPosition = dictionary.get("playerPosition", 0)
        self.playerRole = dictionary.get("playerRole", 0)
        self.quadraKills = dictionary.get("quadraKills", 0)
        self.sightWardsBought = dictionary.get("sightWardsBought", 0)
        self.spell1Cast = dictionary.get("spell1Cast", 0)
        self.spell2Cast = dictionary.get("spell2Cast", 0)
        self.spell3Cast = dictionary.get("spell3Cast", 0)
        self.spell4Cast = dictionary.get("spell4Cast", 0)
        self.summonSpell1Cast = dictionary.get("summonSpell1Cast", 0)
        self.summonSpell2Cast = dictionary.get("summonSpell2Cast", 0)
        self.superMonsterKilled = dictionary.get("superMonsterKilled", 0)
        self.team = dictionary.get("team", 0)
        self.teamObjective = dictionary.get("teamObjective", 0)
        self.timePlayed = dictionary.get("timePlayed", 0)
        self.totalDamageDealt = dictionary.get("totalDamageDealt", 0)
        self.totalDamageDealtToChampions = dictionary.get("totalDamageDealtToChampions", 0)
        self.totalDamageTaken = dictionary.get("totalDamageTaken", 0)
        self.totalHeal = dictionary.get("totalHeal", 0)
        self.totalPlayerScore = dictionary.get("totalPlayerScore", 0)
        self.totalScoreRank = dictionary.get("totalScoreRank", 0)
        self.totalTimeCrowdControlDealt = dictionary.get("totalTimeCrowdControlDealt", 0)
        self.totalUnitsHealed = dictionary.get("totalUnitsHealed", 0)
        self.tripleKills = dictionary.get("tripleKills", 0)
        self.trueDamageDealtPlayer = dictionary.get("trueDamageDealtPlayer", 0)
        self.trueDamageDealtToChampions = dictionary.get("trueDamageDealtToChampions", 0)
        self.trueDamageTaken = dictionary.get("trueDamageTaken", 0)
        self.turretsKilled = dictionary.get("turretsKilled", 0)
        self.unrealKills = dictionary.get("unrealKills", 0)
        self.victoryPointTotal = dictionary.get("victoryPointTotal", 0)
        self.visionWardsBought = dictionary.get("visionWardsBought", 0)
        self.wardKilled = dictionary.get("wardKilled", 0)
        self.wardPlaced = dictionary.get("wardPlaced", 0)
        self.win = dictionary.get("win", False)


@cassiopeia.type.core.common.inheritdocs
class Player(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        championId (int): champion id associated with player
        summonerId (int): summoner id associated with player
        teamId (int): team id associated with player
    """
    def __init__(self, dictionary):
        self.championId = dictionary.get("championId", 0)
        self.summonerId = dictionary.get("summonerId", 0)
        self.teamId = dictionary.get("teamId", 0)


@cassiopeia.type.core.common.inheritdocs
class Game(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Args:
        championId (int): champion ID associated with game
        createDate (int): date that end game data was recorded, specified as epoch milliseconds
        fellowPlayers (list<Player>): other players associated with the game
        gameId (int): game ID
        gameMode (str): game mode (Legal values: CLASSIC, ODIN, ARAM, TUTORIAL, ONEFORALL, ASCENSION, FIRSTBLOOD, KINGPORO)
        gameType (str): game type (Legal values: CUSTOM_GAME, MATCHED_GAME, TUTORIAL_GAME)
        invalid (bool): invalid flag
        ipEarned (int): IP Earned
        level (int): level
        mapId (int): map ID
        spell1 (int): ID of first summoner spell
        spell2 (int): ID of second summoner spell
        stats (RawStats): statistics associated with the game for this summoner
        subType (str): game sub-type (Legal values: NONE, NORMAL, BOT, RANKED_SOLO_5x5, RANKED_PREMADE_3x3, RANKED_PREMADE_5x5, ODIN_UNRANKED, RANKED_TEAM_3x3, RANKED_TEAM_5x5, NORMAL_3x3, BOT_3x3, CAP_5x5, ARAM_UNRANKED_5x5, ONEFORALL_5x5, FIRSTBLOOD_1x1, FIRSTBLOOD_2x2, SR_6x6, URF, URF_BOT, NIGHTMARE_BOT, ASCENSION, HEXAKILL, KING_PORO, COUNTER_PICK)
        teamId (int): team ID associated with game. Team ID 100 is blue team. Team ID 300 is purple team.
    """
    def __init__(self, dictionary):
        self.championId = dictionary.get("championId", 0)
        self.createDate = dictionary.get("createDate", 0)
        self.fellowPlayers = [(Player(player) if not isinstance(player, Player) else player) for player in dictionary.get("fellowPlayers", []) if player]
        self.gameId = dictionary.get("gameId", 0)
        self.gameMode = dictionary.get("gameMode", "")
        self.gameType = dictionary.get("gameType", "")
        self.invalid = dictionary.get("invalid", False)
        self.ipEarned = dictionary.get("ipEarned", 0)
        self.level = dictionary.get("level", 0)
        self.mapId = dictionary.get("mapId", 0)
        self.spell1 = dictionary.get("spell1", 0)
        self.spell2 = dictionary.get("spell2", 0)
        val = dictionary.get("stats", None)
        self.stats = RawStats(val) if val and not isinstance(val, RawStats) else val
        self.subType = dictionary.get("subType", "")
        self.teamId = dictionary.get("teamId", 0)

    @property
    def champion_ids(self):
        """
        Gets all champion IDs contained in this object
        """
        ids = set()
        ids.add(self.championId)
        for p in self.fellowPlayers:
            ids.add(p.championId)
        return ids

    @property
    def summoner_ids(self):
        """
        Gets all champion IDs contained in this object
        """
        ids = set()
        for p in self.fellowPlayers:
            if p.summonerId:
                ids.add(p.summonerId)
        return ids

    @property
    def summoner_spell_ids(self):
        """
        Gets all champion IDs contained in this object
        """
        ids = set()
        ids.add(self.spell1)
        ids.add(self.spell2)
        return ids

    @property
    def item_ids(self):
        """
        Gets all champion IDs contained in this object
        """
        ids = set()
        s = self.stats
        if s.item0:
            ids.add(s.item0)
        if s.item1:
            ids.add(s.item1)
        if s.item2:
            ids.add(s.item2)
        if s.item3:
            ids.add(s.item3)
        if s.item4:
            ids.add(s.item4)
        if s.item5:
            ids.add(s.item5)
        if s.item6:
            ids.add(s.item6)
        return ids


@cassiopeia.type.core.common.inheritdocs
class RecentGames(cassiopeia.type.dto.common.CassiopeiaDto):
    """
    Gets all champion IDs contained in this object
    """
    def __init__(self, dictionary):
        self.games = [(Game(game) if not isinstance(game, Game) else game) for game in dictionary.get("games", []) if game]
        self.summonerId = dictionary.get("summonerId", 0)

    @property
    def champion_ids(self):
        """
        Gets all summoner IDs contained in this object
        """
        ids = set()
        for game in self.games:
            ids |= game.champion_ids
        return ids

    @property
    def summoner_ids(self):
        """
        Gets all summoner IDs contained in this object
        """
        ids = set()
        ids.add(self.summonerId)
        for game in self.games:
            ids |= game.summoner_ids
        return ids

    @property
    def summoner_spell_ids(self):
        """
        Gets all summoner IDs contained in this object
        """
        ids = set()
        for game in self.games:
            ids |= game.summoner_spell_ids
        return ids

    @property
    def item_ids(self):
        """
        Gets all summoner IDs contained in this object
        """
        ids = set()
        for game in self.games:
            ids |= game.item_ids
        return ids


###############################
# Dynamic SQLAlchemy bindings #
###############################
def _sa_bind_raw_stats():
    global RawStats

    @cassiopeia.type.core.common.inheritdocs
    class RawStats(RawStats, cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "GameRawStats"
        assists = sqlalchemy.Column(sqlalchemy.Integer)
        barracksKilled = sqlalchemy.Column(sqlalchemy.Integer)
        championsKilled = sqlalchemy.Column(sqlalchemy.Integer)
        combatPlayerScore = sqlalchemy.Column(sqlalchemy.Integer)
        consumablesPurchased = sqlalchemy.Column(sqlalchemy.Integer)
        damageDealtPlayer = sqlalchemy.Column(sqlalchemy.Integer)
        doubleKills = sqlalchemy.Column(sqlalchemy.Integer)
        firstBlood = sqlalchemy.Column(sqlalchemy.Integer)
        gold = sqlalchemy.Column(sqlalchemy.Integer)
        goldEarned = sqlalchemy.Column(sqlalchemy.Integer)
        goldSpent = sqlalchemy.Column(sqlalchemy.Integer)
        item0 = sqlalchemy.Column(sqlalchemy.Integer)
        item1 = sqlalchemy.Column(sqlalchemy.Integer)
        item2 = sqlalchemy.Column(sqlalchemy.Integer)
        item3 = sqlalchemy.Column(sqlalchemy.Integer)
        item4 = sqlalchemy.Column(sqlalchemy.Integer)
        item5 = sqlalchemy.Column(sqlalchemy.Integer)
        item6 = sqlalchemy.Column(sqlalchemy.Integer)
        itemsPurchased = sqlalchemy.Column(sqlalchemy.Integer)
        killingSprees = sqlalchemy.Column(sqlalchemy.Integer)
        largestCriticalStrike = sqlalchemy.Column(sqlalchemy.Integer)
        largestKillingSpree = sqlalchemy.Column(sqlalchemy.Integer)
        largestMultiKill = sqlalchemy.Column(sqlalchemy.Integer)
        legendaryItemsCreated = sqlalchemy.Column(sqlalchemy.Integer)
        level = sqlalchemy.Column(sqlalchemy.Integer)
        magicDamageDealtPlayer = sqlalchemy.Column(sqlalchemy.Integer)
        magicDamageDealtToChampions = sqlalchemy.Column(sqlalchemy.Integer)
        magicDamageTaken = sqlalchemy.Column(sqlalchemy.Integer)
        minionsDenied = sqlalchemy.Column(sqlalchemy.Integer)
        minionsKilled = sqlalchemy.Column(sqlalchemy.Integer)
        neutralMinionsKilled = sqlalchemy.Column(sqlalchemy.Integer)
        neutralMinionsKilledEnemyJungle = sqlalchemy.Column(sqlalchemy.Integer)
        neutralMinionsKilledYourJungle = sqlalchemy.Column(sqlalchemy.Integer)
        nexusKilled = sqlalchemy.Column(sqlalchemy.Boolean)
        nodeCapture = sqlalchemy.Column(sqlalchemy.Integer)
        nodeCaptureAssist = sqlalchemy.Column(sqlalchemy.Integer)
        nodeNeutralize = sqlalchemy.Column(sqlalchemy.Integer)
        nodeNeutralizeAssist = sqlalchemy.Column(sqlalchemy.Integer)
        numDeaths = sqlalchemy.Column(sqlalchemy.Integer)
        numItemsBought = sqlalchemy.Column(sqlalchemy.Integer)
        objectivePlayerScore = sqlalchemy.Column(sqlalchemy.Integer)
        pentaKills = sqlalchemy.Column(sqlalchemy.Integer)
        physicalDamageDealtPlayer = sqlalchemy.Column(sqlalchemy.Integer)
        physicalDamageDealtToChampions = sqlalchemy.Column(sqlalchemy.Integer)
        physicalDamageTaken = sqlalchemy.Column(sqlalchemy.Integer)
        playerPosition = sqlalchemy.Column(sqlalchemy.Integer)
        playerRole = sqlalchemy.Column(sqlalchemy.Integer)
        quadraKills = sqlalchemy.Column(sqlalchemy.Integer)
        sightWardsBought = sqlalchemy.Column(sqlalchemy.Integer)
        spell1Cast = sqlalchemy.Column(sqlalchemy.Integer)
        spell2Cast = sqlalchemy.Column(sqlalchemy.Integer)
        spell3Cast = sqlalchemy.Column(sqlalchemy.Integer)
        spell4Cast = sqlalchemy.Column(sqlalchemy.Integer)
        summonSpell1Cast = sqlalchemy.Column(sqlalchemy.Integer)
        summonSpell2Cast = sqlalchemy.Column(sqlalchemy.Integer)
        superMonsterKilled = sqlalchemy.Column(sqlalchemy.Integer)
        team = sqlalchemy.Column(sqlalchemy.Integer)
        teamObjective = sqlalchemy.Column(sqlalchemy.Integer)
        timePlayed = sqlalchemy.Column(sqlalchemy.Integer)
        totalDamageDealt = sqlalchemy.Column(sqlalchemy.Integer)
        totalDamageDealtToChampions = sqlalchemy.Column(sqlalchemy.Integer)
        totalDamageTaken = sqlalchemy.Column(sqlalchemy.Integer)
        totalHeal = sqlalchemy.Column(sqlalchemy.Integer)
        totalPlayerScore = sqlalchemy.Column(sqlalchemy.Integer)
        totalScoreRank = sqlalchemy.Column(sqlalchemy.Integer)
        totalTimeCrowdControlDealt = sqlalchemy.Column(sqlalchemy.Integer)
        totalUnitsHealed = sqlalchemy.Column(sqlalchemy.Integer)
        tripleKills = sqlalchemy.Column(sqlalchemy.Integer)
        trueDamageDealtPlayer = sqlalchemy.Column(sqlalchemy.Integer)
        trueDamageDealtToChampions = sqlalchemy.Column(sqlalchemy.Integer)
        trueDamageTaken = sqlalchemy.Column(sqlalchemy.Integer)
        turretsKilled = sqlalchemy.Column(sqlalchemy.Integer)
        unrealKills = sqlalchemy.Column(sqlalchemy.Integer)
        victoryPointTotal = sqlalchemy.Column(sqlalchemy.Integer)
        visionWardsBought = sqlalchemy.Column(sqlalchemy.Integer)
        wardKilled = sqlalchemy.Column(sqlalchemy.Integer)
        wardPlaced = sqlalchemy.Column(sqlalchemy.Integer)
        win = sqlalchemy.Column(sqlalchemy.Boolean)
        _id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
        _game_id = sqlalchemy.Column(sqlalchemy.Integer, sqlalchemy.ForeignKey("Game.gameId", ondelete="CASCADE"))


def _sa_bind_player():
    global Player

    @cassiopeia.type.core.common.inheritdocs
    class Player(Player, cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "GamePlayer"
        championId = sqlalchemy.Column(sqlalchemy.Integer)
        summonerId = sqlalchemy.Column(sqlalchemy.Integer)
        teamId = sqlalchemy.Column(sqlalchemy.Integer)
        _id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
        _game_id = sqlalchemy.Column(sqlalchemy.Integer, sqlalchemy.ForeignKey("Game.gameId", ondelete="CASCADE"))


def _sa_bind_game():
    global Game

    @cassiopeia.type.core.common.inheritdocs
    class Game(Game, cassiopeia.type.dto.common.BaseDB):
        __tablename__ = "Game"
        championId = sqlalchemy.Column(sqlalchemy.Integer)
        createDate = sqlalchemy.Column(sqlalchemy.BigInteger)
        fellowPlayers = sqlalchemy.orm.relationship("cassiopeia.type.dto.game.Player", cascade="all, delete-orphan", passive_deletes=True)
        gameId = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
        gameMode = sqlalchemy.Column(sqlalchemy.String(30))
        gameType = sqlalchemy.Column(sqlalchemy.String(30))
        invalid = sqlalchemy.Column(sqlalchemy.Boolean)
        ipEarned = sqlalchemy.Column(sqlalchemy.Integer)
        level = sqlalchemy.Column(sqlalchemy.Integer)
        mapId = sqlalchemy.Column(sqlalchemy.Integer)
        spell1 = sqlalchemy.Column(sqlalchemy.Integer)
        spell2 = sqlalchemy.Column(sqlalchemy.Integer)
        stats = sqlalchemy.orm.relationship("cassiopeia.type.dto.game.RawStats", uselist=False, cascade="all, delete-orphan", passive_deletes=True)
        subType = sqlalchemy.Column(sqlalchemy.String(30))
        teamId = sqlalchemy.Column(sqlalchemy.Integer)


def _sa_bind_all():
    _sa_bind_raw_stats()
    _sa_bind_player()
    _sa_bind_game()



from cassiopeia import baseriotapi
from .. import int_test_handler


def test_tracking_on_valid_request():
    """Checks that the request tracking works on requests with expected return code of 200"""
    start_count = baseriotapi.get_requests_count()
    int_test_handler.test_result(baseriotapi.get_match(int_test_handler.match_id))
    int_test_handler.test_result(baseriotapi.get_requests_count() == (start_count[0] + 1, start_count[1] + 1))


def test_tracking_on_404_request():
    """Checks that the request tracking works on requests which raise exceptions"""
    start_count = baseriotapi.get_requests_count()
    try:
        baseriotapi.get_champion(int_test_handler.non_existent_champion_id)
    except:
        pass
    int_test_handler.test_result(baseriotapi.get_requests_count() == (start_count[0], start_count[1] + 1))



from cassiopeia import baseriotapi
from .. import int_test_handler


def test_all():
    print("dto/matchlistapi tests...")
    test_match_list()


def test_match_list():
    int_test_handler.test_result(baseriotapi.get_match_list(int_test_handler.summoner_id))
